@@ -1,13 +1,13 @@
 package Code::TidyAll::Test::Plugin::RepeatFoo;
 BEGIN {
-  $Code::TidyAll::Test::Plugin::RepeatFoo::VERSION = '0.02';
+  $Code::TidyAll::Test::Plugin::RepeatFoo::VERSION = '0.03';
 }
 use Code::TidyAll::Util qw(read_file write_file);
 use base qw(Code::TidyAll::Plugin);
 use strict;
 use warnings;
 
-sub process_source {
+sub transform_source {
     my ( $self, $source ) = @_;
     my $times = $self->options->{times} || die "no times specified";
     return $source x $times;
