@@ -1,7 +1,8 @@
 package Code::TidyAll::Util;
 BEGIN {
-  $Code::TidyAll::Util::VERSION = '0.02';
+  $Code::TidyAll::Util::VERSION = '0.03';
 }
+use Cwd qw(realpath);
 use Data::Dumper;
 use File::Basename;
 use File::Path;
@@ -15,7 +16,7 @@ use warnings;
 use base qw(Exporter);
 
 our @EXPORT_OK =
-  qw(abs2rel basename can_load dirname dump_one_line mkpath read_dir read_file rel2abs tempdir_simple uniq write_file );
+  qw(abs2rel basename can_load dirname dump_one_line mkpath read_dir read_file realpath rel2abs tempdir_simple uniq write_file );
 
 sub can_load {
 
@@ -43,7 +44,7 @@ sub can_load {
 
 sub tempdir_simple {
     my $template = shift || 'Code-TidyAll-XXXX';
-    return tempdir( $template, TMPDIR => 1, CLEANUP => 1 );
+    return tempdir( $template, TMPDIR => 1, CLEANUP => 0 );
 }
 
 sub dump_one_line {
